using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;

using SubSystems.TE;
using SubSystems.HT;

namespace Demo
{
	/// <summary>
 /// Summary description for DmoTe.
	/// </summary>
 internal class DmoTe : System.Windows.Forms.Form
	{
  private Demo dm;
  private int  CurCtl;
  
  private Tern toc = null;
  private Htn  htn = null;

  /// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

  public DmoTe(Demo parent)
		{
    dm=parent;           // set the parent class
    CurCtl=dm.CurCtl;

			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
     
   // free up the open slot
   dm.ctl[CurCtl].InUse=false;
   dm.ctl[CurCtl].toc=null;
   dm.ctl[CurCtl].form=null;  
   
   if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// DmoTe
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(240, 173);
			this.Name = "DmoTe";
			this.Text = "title";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.DmoTe_Closing);
			this.Load += new System.EventHandler(this.DmoTe_Load);

		}
		#endregion

  private void DmoTe_Load(object sender, System.EventArgs ev)
  {
     Demo.StrCtl param=dm.ctl[CurCtl];   // make a local copy of the parameter structure

     // Set the form dimensions
     this.Location=new Point(param.x,param.y);
     this.Size=new Size(param.width,param.height);
     this.Text="Unknown";

     // Create te control
     toc = new Tern();

     toc.Location=new Point(0,0);
     toc.Size=new Size(this.ClientRectangle.Width,this.ClientRectangle.Height);

     toc.WordWrap=true;            // Word wrap
     toc.PageMode=true;            // page mode
     toc.PrintViewMode=true;  // print-view mode
     toc.FittedView=param.FittedView;        // fitted view mode
     toc.ShowRuler=false;          // show the ruler
     toc.ShowStatusBar=true;  // show the status bar
     toc.ShowToolBar=false;      // show the toolbar
     toc.VertScrollBar=param.VertScrollBar;  // show vertical toolbar
     toc.HorzScrollBar=param.HorzScrollBar;  // show the horizontal scroll bar
     toc.BorderMargin=param.BorderMargin;    // create the border margin
     toc.ReadOnlyMode=!param.EditMode;           // read-only mode

     toc.Parent=this;

     // Set Te flags
     toc.TerSetFlags5(true,tc.TFLAG5_SET_FORM_TITLE);  // set form title automatically when a new file is loaded in the editor
     
     toc.Show();
     
     param.toc=toc;
     param.form=this;
     
     param.htn=htn=new Htn(toc);
     
     // set events
     param.htn.Link+=new Htn.EventLink(EvLink);
     param.htn.Form+=new Htn.EventForm(EvForm);
     param.htn.Close+=new Htn.EventClose(EvClose);
     
     if (param.ShowMenu) this.Menu=htn.BuildMenu();
     
     // read the file into the control
     if (param.file.Length>0) {
        int len=param.file.Length;
        string pFileExt="";
        bool IsHtmlFile=false,result=false;

        if (len>=4) pFileExt=param.file.Substring(len-4);
        if (String.Compare(pFileExt,".HTM",true)==0) IsHtmlFile=true;
        if (htn.HtsIsHttpFile(param.file)) IsHtmlFile=true;

        if (IsHtmlFile) {  // Read html data into the control
           if (param.InputType=='F') result=htn.HtsRead(hc.HTML_FILE,param.file,null,null);
           else {
              int size;
              char[] data=toc.TerFileToMem(param.file,out size);
              if (size>0) {
                string HtmlText=new string(data,0,size);
                result=htn.HtsRead(hc.HTML_BUF,"",HtmlText,null);
              }
           }
        }
        else {             // possibly an RTF file, read it directly into TER dll
           if (param.InputType=='F') result=toc.ReadTerFile(param.file);
           else {
              int size;
              char[] data=toc.TerFileToMem(param.file,out size);
              if (size>0) {
                string RtfText=new string(data,0,size);
                result=toc.SetTerBuffer(RtfText,param.file);
              }
           }
        }

        if (!result) MessageBox.Show("File not found or file read error!",null,MessageBoxButtons.OK);
     }

     dm.ctl[CurCtl]=param;                  // pass the updated structure
  }
  
  /****************************************************************************************************
  EvLink:
  This event is fired when the user clicks on a hyperlink in HTML Add-on
  ****************************************************************************************************/
  protected void EvLink(object Sender, string path)
  {
     string dest="",FilePath;
     int len,i,MouseX,MouseY;
     string str,MousePos="";
     bool FullPath;
     bool IsInternetFile,IsMailTo;
     string CurPath;
     Htn htn=(Htn)Sender;

     htn.HtsGetCurPath(out FilePath);           // path for the currently loaded file
     IsInternetFile=(htn.HtsIsHttpFile(path) || htn.HtsIsHttpFile(FilePath));
    
     IsMailTo=path.StartsWith("mailto:")?true:false;
    
     if (IsMailTo) {
        MessageBox.Show(path,"Mail-to link",MessageBoxButtons.OK);
        return;
     } 

     if (!IsInternetFile) {
        // Extract the mouse position (stored as ?X,Y) from the path name
        len=path.Length;
        for (i=len-1;i>=0;i--) if (path[i]=='?') break;

        if (i>=0) {
           MousePos=path.Substring(i+1);
           path=path.Substring(0,i);

           // Extract mouse x and y position
           for (i=0;i<MousePos.Length;i++) if (MousePos[i]==',') break;
           if (i<MousePos.Length) {     // ?X,Y str found
              MouseX=ToInt(MousePos.Substring(0,i));    // not used in this example
              MouseY=ToInt(MousePos.Substring(i+1));
              str="x: "+MouseX.ToString()+" y: "+MouseY.ToString();
              MessageBox.Show(str,"Pixel Position",MessageBoxButtons.OK);
              if (path.Length==0) return;  // no link file specified
           }
        }

        // Extract destination name from the path name
        len=path.Length;
        for (i=len-1;i>=0;i--) if (path[i]=='#') break;

        if (i>0) {
           dest=path.Substring(i+1);
           path=path.Substring(0,i);
        }

        // check if full path specified
        FullPath=true;
        len=path.Length;
        for (i=0;i<len;i++) if (path[i]=='\\' || path[i]==':') break;
        if (i==len) FullPath=false;


        // check if file exists
        if (FullPath) str=path;  // full path in the path str
        else {                               // take the directory from the initial html file
           htn.HtsGetCurPath(out CurPath);
           str=CurPath;
           str+=path;
        }
        path=str;            // return the full path of the picture file
        if (!File.Exists(path)) {      // if a new file
           MessageBox.Show("File Not Found!",path,MessageBoxButtons.OK);
           return;
        }
     }

     // read the html file
     htn.HtsClearWindow();         // clear the existing text

     if (!htn.HtsRead(hc.HTML_FILE,path,null,dest)) {
        MessageBox.Show("File not found or file read error!",null,MessageBoxButtons.OK);
     }
  } 

  /****************************************************************************************************
  EvForm:
  This event is fired the user submits an input form.  Here we simply display the form field data.
  ****************************************************************************************************/
  protected void EvForm(object Sender, int FormNo)
  {
     int  TotalFields,i,IntData,FieldType;
     string action,method,name,FormData="",TextData;

     if ((TotalFields=htn.HtsGetFormInfo(FormNo,out action,out method))<0) return;

     FormData="Form No: "+FormNo.ToString()+" Action: "+action+" Method: "+method+". ";

     // get the name of the submit field
     if (htn.HtsGetFieldData(-1,-1,out name,out IntData,out TextData)<0) return;
     FormData+="Submit Field Name: "+name+". ";

     // Extract each field
     for (i=0;i<TotalFields;i++) {
        // extract field type, field name, integer data, and text data
        if ((FieldType=htn.HtsGetFieldData(FormNo,i,out name,out IntData,out TextData))<0) continue;

        // append common field data to the display buffer
        FormData+="\r\n";
        FormData+="Field# "+i.ToString()+" name: "+name+", ";

        // process data for each field type
        if (FieldType==hc.FTYPE_TEXT) {
           // append text data to the display buffer
           FormData+="type: Text, data: "+TextData+"; ";
        }
        else if (FieldType==hc.FTYPE_RADIO) {
           // append text data to the display buffer
           FormData+="type: Radio, ";
           if (TextData.Length>0) FormData+="val: "+TextData+", ";
           if (IntData!=0) FormData+="Checked; ";
           else            FormData+="Unchecked; ";
        }
        else if (FieldType==hc.FTYPE_SUBMIT) {
           // append field type to the display buffer
           FormData+="type: Submit; value: "+TextData+"; ";
        }
        else if (FieldType==hc.FTYPE_RESET) {
           // append field type to the display buffer
           FormData+="type: Reset; ";
        }
        else if (FieldType==hc.FTYPE_TEXTAREA) {
           // append textarea data to the display buffer
           FormData+="type: Textarea, data: "+TextData+"; ";
        }
        else if (FieldType==hc.FTYPE_CHECKBOX) {
           // append text data to the display buffer
           if (IntData!=0) FormData+="type: Checkbox, Checked; ";
           else            FormData+="type: Checkbox, Unchecked; ";
        }
        else if (FieldType==hc.FTYPE_HIDDEN) {
           // append hidden data to the display buffer
           FormData+="type: Hidden, value: "+TextData+"; ";
        }
        else if (FieldType==hc.FTYPE_PASSWORD) {
           // append text data to the display buffer
           FormData+="type: Password, data: "+TextData+"; ";
        }
        else if (FieldType==hc.FTYPE_IMAGE) {
           // append pixel location to the display buffer
           FormData+="type: Image, x/y: "+TextData+"; ";
        }
        else if (FieldType==hc.FTYPE_SELECT) {
           // append the selected items to the display buffer
           FormData+="type: Select, item(s): "+TextData+"; ";
        }
     }

     MessageBox.Show(FormData,"Submitted Form Data",MessageBoxButtons.OK);
  }

  /****************************************************************************************************
  EvClose:
  This event is fired when the user select 'Exit' from the menu.
  ****************************************************************************************************/
  protected void EvClose(object Sender, string FileName)
  {
     Demo.StrCtl param=dm.ctl[CurCtl];   // make a local copy of the parameter structure
     string OutBuf;
     bool closed=false;

     if (param.InputType=='F') closed=param.htn.HtsClose(hc.HTML_FILE,FileName,out OutBuf);
     else                      closed=param.htn.HtsClose(hc.HTML_BUF,FileName,out OutBuf);  

     if (closed) {
        dm.ctl[CurCtl].InUse=false;
        dm.ctl[CurCtl].form.Dispose();  // close the form hosting the Tern control

        dm.ctl[CurCtl].form=null;
        dm.ctl[CurCtl].toc=null;
        dm.ctl[CurCtl].htn=null;
     } 
  } 

  protected override void OnResize(EventArgs ev)
  {
    if (toc!=null) {
      toc.Location=new Point(0,0);
      toc.Size=new Size(this.ClientRectangle.Width,this.ClientRectangle.Height);
    }
  }

  protected bool MergeData(object Sender, string name, out string data)   // testing of  MergeData event
  {
      data="";
      if (name=="date") {
         data=DateTime.Today.Date.ToString();
         return true;
      }
      return false; 
  } 

  private void DmoTe_Closing(object sender, System.ComponentModel.CancelEventArgs ev)
	 {
     if (!toc.TerQueryExit()) {
       ev.Cancel=true;
       return;
     }
     
     toc.Dispose();
  }

  /***************************************************************************
      ToInt:
      Convert string to integer
  ****************************************************************************/
  internal new int ToInt(TextBox item)
  {
     return ToInt(item.Text);
  }
  internal new int ToInt(string txt)
  {
     bool SkipBool;
     return ToInt(txt,out SkipBool);
  }
   
  internal new int ToInt(string txt, out bool error)
  {
     error=false;
     try {
        return (int)Convert.ToInt32(txt);
     } 
     catch (System.Exception)
     {
        error=true;
        //e.ExpMessage=ev.Message;
        return 0;
     }
  } 

	}
}
